@@ -29,7 +29,6 @@
 #include <cassert>
 #include <cstddef>
 #include <cstring>
-#include <cstdint>
 #include <filesystem>
 #include <optional>
 #include <string>
@@ -93,7 +92,7 @@
 #define FASTGLTF_QUOTE(x) FASTGLTF_QUOTE_Q(x)
 
 // fastgltf version string. Use FASTGLTF_QUOTE to stringify.
-#define FASTGLTF_VERSION 0.7.0
+#define FASTGLTF_VERSION 0.7.1
 
 namespace fastgltf {
 #if defined(FASTGLTF_USE_64BIT_FLOAT) && FASTGLTF_USE_64BIT_FLOAT
@@ -329,14 +328,14 @@ namespace fastgltf {
     }
 
     constexpr auto getElementByteSize(AccessorType type, ComponentType componentType) noexcept {
-        const auto componentSize = getComponentBitSize(componentType) / 8;
+        const auto componentSize = getComponentBitSize(componentType) / 8U;
         auto numComponents = getNumComponents(type);
         const auto rowCount = getElementRowCount(type);
         if (isMatrix(type) && (rowCount * componentSize) % 4 != 0) {
             // Matrices need extra padding per-column which affects their size.
             numComponents += rowCount * (4 - (rowCount % 4));
         }
-        return static_cast<std::uint16_t>(numComponents) * componentSize;
+        return static_cast<std::size_t>(numComponents * componentSize);
     }
 
     /**
@@ -1577,7 +1576,7 @@ namespace fastgltf {
             }
             return instancingAttributes.cend();
         }
-   };
+    };
 
     struct Primitive {
 		using attribute_type = std::pair<FASTGLTF_STD_PMR_NS::string, std::size_t>;
@@ -1592,6 +1591,13 @@ namespace fastgltf {
         Optional<std::size_t> indicesAccessor;
         Optional<std::size_t> materialIndex;
 
+		/**
+		 * Represents the mappings data from KHR_material_variants.
+		 * Use the variant index to index into this array to get the corresponding material index to use.
+		 * If the optional has no value, the normal materialIndex should be used as a fallback.
+		 */
+		std::vector<Optional<std::size_t>> mappings;
+
 		[[nodiscard]] auto findAttribute(std::string_view name) noexcept {
 			for (auto it = attributes.begin(); it != attributes.end(); ++it) {
 				if (it->first == name)
@@ -2025,6 +2031,8 @@ namespace fastgltf {
         std::vector<Skin> skins;
         std::vector<Texture> textures;
 
+		std::vector<std::string> materialVariants;
+
         // Keeps tracked of categories that were actually parsed.
         Category availableCategories = Category::None;
 
@@ -2052,6 +2060,7 @@ namespace fastgltf {
 				scenes(std::move(other.scenes)),
 				skins(std::move(other.skins)),
 				textures(std::move(other.textures)),
+				materialVariants(std::move(other.materialVariants)),
 				availableCategories(other.availableCategories) {}
 
 		Asset& operator=(const Asset& other) = delete;
@@ -2077,6 +2086,7 @@ namespace fastgltf {
 			scenes = std::move(other.scenes);
 			skins = std::move(other.skins);
 			textures = std::move(other.textures);
+			materialVariants = std::move(other.materialVariants);
 			availableCategories = other.availableCategories;
 			return *this;
 		}
