@@ -18,6 +18,7 @@
 #include <fastgltf/core.hpp>
 #include <fastgltf/types.hpp>
 #include "gltf_path.hpp"
+#include <simdjson.h>
 
 constexpr auto noOptions = fastgltf::Options::None;
 
@@ -86,6 +87,11 @@ TEST_CASE("Test extension stringification", "[gltf-loader]") {
 
 	stringified = stringifyExtension(fastgltf::Extensions::EXT_meshopt_compression | fastgltf::Extensions::EXT_texture_webp);
 	REQUIRE(stringified == fastgltf::extensions::EXT_meshopt_compression);
+
+	auto list = stringifyExtensionBits(fastgltf::Extensions::EXT_meshopt_compression | fastgltf::Extensions::EXT_texture_webp);
+	REQUIRE(list.size() == 2);
+	REQUIRE(list[0] == fastgltf::extensions::EXT_meshopt_compression);
+	REQUIRE(list[1] == fastgltf::extensions::EXT_texture_webp);
 }
 
 TEST_CASE("Test if glTF type detection works", "[gltf-loader]") {
@@ -586,3 +592,78 @@ TEST_CASE("Test unicode characters", "[gltf-loader]") {
 	auto bufferUri = std::get<fastgltf::sources::URI>(asset->buffers[0].data);
 	REQUIRE(bufferUri.uri.path() == reinterpret_cast<const char*>(u8"Unicode❤♻Binary.bin"));
 }
+
+TEST_CASE("Test extras callback", "[gltf-loader]") {
+	auto materialVariants = sampleModels / "2.0" / "MaterialsVariantsShoe" / "glTF";
+	fastgltf::GltfDataBuffer jsonData;
+	REQUIRE(jsonData.loadFromFile(materialVariants / "MaterialsVariantsShoe.gltf"));
+
+	std::vector<std::string> nodeNames;
+
+	// lambda callback to parse the glTF JSON from the jsonData buffer
+	auto parseJson = [&]() {
+		auto extrasCallback = [](simdjson::dom::object *extras, std::size_t objectIndex, fastgltf::Category category,
+								 void *userPointer) {
+			if (category != fastgltf::Category::Nodes)
+				return;
+			auto *nodeNames = static_cast<std::vector<std::string> *>(userPointer);
+			nodeNames->resize(fastgltf::max(nodeNames->size(), objectIndex + 1));
+
+			std::string_view nodeName;
+			if ((*extras)["name"].get_string().get(nodeName) == simdjson::SUCCESS) {
+				(*nodeNames)[objectIndex] = std::string(nodeName);
+			}
+		};
+
+		fastgltf::Parser parser;
+		parser.setExtrasParseCallback(extrasCallback);
+		parser.setUserPointer(&nodeNames);
+		return parser.loadGltfJson(&jsonData, materialVariants);
+	};
+
+	// The asset has to be reused for exporting in the next section.
+	auto asset = parseJson();
+	SECTION("Validate node names from extras") {
+		REQUIRE(asset.error() == fastgltf::Error::None);
+		REQUIRE(fastgltf::validate(asset.get()) == fastgltf::Error::None);
+
+		REQUIRE(nodeNames.size() == 3);
+		REQUIRE(nodeNames.size() == asset->nodes.size());
+
+		REQUIRE(nodeNames[0] == "Shoe");
+		REQUIRE(nodeNames[1] == "g Shoe");
+		REQUIRE(nodeNames[2] == "Shoe.obj");
+	}
+
+	SECTION("Re-export asset") {
+		auto extrasWriteCallback = [](std::size_t objectIndex, fastgltf::Category category,
+									  void *userPointer) -> std::optional<std::string> {
+			if (category != fastgltf::Category::Nodes)
+				return std::nullopt;
+
+			auto *nodeNames = static_cast<std::vector<std::string> *>(userPointer);
+			if (objectIndex >= nodeNames->size())
+				return std::nullopt; // TODO: Error?
+			return {std::string(R"({"name":")") + (*nodeNames)[objectIndex] + "\"}"};
+		};
+		fastgltf::Exporter exporter;
+		exporter.setUserPointer(&nodeNames);
+		exporter.setExtrasWriteCallback(extrasWriteCallback);
+		auto json = exporter.writeGltfJson(asset.get());
+		REQUIRE(json.error() == fastgltf::Error::None);
+
+		// Update the data buffer
+		auto& string = json.get().output;
+		jsonData.fromByteView(reinterpret_cast<std::uint8_t*>(string.data()), string.size(), string.capacity());
+
+		nodeNames.clear();
+		auto reparsed = parseJson();
+		REQUIRE(reparsed.error() == fastgltf::Error::None);
+		REQUIRE(fastgltf::validate(reparsed.get()) == fastgltf::Error::None);
+
+		REQUIRE(nodeNames.size() == 3);
+		REQUIRE(nodeNames[0] == "Shoe");
+		REQUIRE(nodeNames[1] == "g Shoe");
+		REQUIRE(nodeNames[2] == "Shoe.obj");
+	}
+}
